import contextlib

from django.conf import settings

from edc_dashboard.middleware_mixins import EdcTemplateMiddlewareMixin

from .dashboard_templates import dashboard_templates


class DashboardMiddleware(EdcTemplateMiddlewareMixin):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        self.check_for_required_request_attrs(request)
        return self.get_response(request)

    def process_view(self, request, *args) -> None:
        template_data = dashboard_templates
        with contextlib.suppress(AttributeError):
            template_data.update(settings.REVIEW_DASHBOARD_BASE_TEMPLATES)
        request.template_data.update(**template_data)

    def process_template_response(self, request, response):
        if getattr(response, "context_data", None):
            response.context_data.update(**request.template_data)
            request.template_data.update(**request.template_data)
        return response



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from django.db import models, transaction

from ...constants import CRF, KEYED, REQUIRED, REQUISITION
from ...metadata import (
    CrfMetadataGetter,
    DeleteMetadataError,
    Destroyer,
    Metadata,
    RequisitionMetadataGetter,
)
from ...metadata_rules import MetadataRuleEvaluator

if TYPE_CHECKING:
    from edc_visit_schedule.visit import Visit
    from edc_visit_tracking.typing_stubs import RelatedVisitProtocol
else:

    class RelatedVisitProtocol: ...


class CreatesMetadataModelMixin(RelatedVisitProtocol, models.Model):
    """A model mixin for visit models to enable them to
    create metadata on save.
    """

    metadata_cls: type[Metadata] = Metadata
    metadata_destroyer_cls: type[Destroyer] = Destroyer
    metadata_rule_evaluator_cls: type[MetadataRuleEvaluator] = MetadataRuleEvaluator

    def metadata_create(self) -> None:
        """Creates metadata, called by post_save signal."""
        metadata = self.metadata_cls(related_visit=self, update_keyed=True)
        metadata.prepare()

    def run_metadata_rules(self, allow_create: bool | None = None) -> None:
        """Runs all the metadata rules.

        Initially called by post_save signal.

        Also called by post_save signal after metadata is updated.
        """
        return self.metadata_rule_evaluator_cls(
            related_visit=self, allow_create=allow_create
        ).evaluate_rules()

    @property
    def metadata_query_options(self) -> dict[str, Any]:
        """Returns a dictionary of query options needed select
        the related_visit.
        """
        visit: Visit = self.visits.get(self.appointment.visit_code)
        return dict(
            visit_schedule_name=self.appointment.visit_schedule_name,
            schedule_name=self.appointment.schedule_name,
            visit_code=visit.code,
            visit_code_sequence=self.appointment.visit_code_sequence,
            timepoint=self.appointment.timepoint,
        )

    @property
    def crf_metadata(self):
        return self.metadata[CRF]

    @property
    def requisition_metadata(self):
        return self.metadata[REQUISITION].filter(entry_status__in=[KEYED, REQUIRED])

    @property
    def crf_metadata_required(self):
        return self.metadata[CRF].filter(entry_status__in=[KEYED, REQUIRED])

    @property
    def metadata(self) -> dict:
        """Returns a dictionary of metadata querysets for each
        metadata category (CRF or REQUISITION).
        """
        metadata = {}
        getter = CrfMetadataGetter(self.appointment)
        metadata[CRF] = getter.metadata_objects
        getter = RequisitionMetadataGetter(self.appointment)
        metadata[REQUISITION] = getter.metadata_objects
        return metadata

    def metadata_delete_for_visit(self) -> None:
        """Deletes metadata for a visit when the visit is deleted.

        See signals.
        """
        with transaction.atomic():
            for key in [CRF, REQUISITION]:
                if [obj for obj in self.metadata[key] if obj.get_entry_status() == KEYED]:
                    raise DeleteMetadataError(
                        f"Metadata cannot be deleted. {key}s have been keyed. Got {self!r}."
                    )
            destroyer = self.metadata_destroyer_cls(related_visit=self)
            destroyer.delete(entry_status_not_in=[KEYED])

    class Meta:
        abstract = True



# Generated by Django 2.0.1 on 2018-01-16 13:28

import django.contrib.sites.managers
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_locator", "0005_auto_20180116_1411")]

    operations = [
        migrations.AlterModelManagers(
            name="subjectlocator",
            managers=[("on_site", django.contrib.sites.managers.CurrentSiteManager())],
        )
    ]



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_dx_review", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="diagnosislocations",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicaldiagnosislocations",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicalreasonsfortesting",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="reasonsfortesting",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



from django.apps import AppConfig as DjangoAppConfig
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_navbar"
    verbose_name = "Edc Navbar"
    register_default_navbar = True
